require 'fileutils'

module AssetZipImportable
  module Importable
    extend ActiveSupport::Concern

    included do
      include Completions
      include Extractions
      include FilePaths

      validates :zip, presence: true

      mount_uploader :zip, ZipUploader
    end

    module ClassMethods
      def import_for(model_class, attr_name)
        define_method :model_class do
          model_class
        end

        define_method :attr_name do
          attr_name
        end
      end
    end

    def import!(callbacks)
      download
      extract

      model_class.transaction do
        extract_files.each do |file|
          @resource = model_class.new(attr_name => file)
          if @resource.save!
            increment_completed_number!
            callbacks[:success].try(:call, @resource)
          end
        end
      end
    rescue => e
      callbacks[:error].try(:call, @resource)
      raise e
    end
  end
end
